#
# Script version 1.0
#
# Version history:
#  1.0  First version of this script
#
#
######################
# Overview:
######################
#
# If the called number is a specific value, it will play an announcement and waits to capture digits. 
# 
# Call 1-800-999-4444
# Hear tone
# Enter 2 digit switch number optionally followed by #
# Hear tone
# Enter TG number – can be 4 or 5 digits depending on which switch the call goes to
# Hear tone
# Enter destination number
# The call then routes to a TG (NAP) based on the 2 digit switch number and the INVITE contains the TG number in the TGRP= parameter.  
# Sample INVITE with TGRP=  INVITE sip:18003608022@192.168.101.180;tgrp=98741 SIP/2.0
#
#Instructions to install the capture_dtmf.rb script
# In order to use this module, follow the steps below:
# 1- Upload new script
#   Gateway -> Routing Scripts -> Import Script File
#     File: capture_dtmf.rb
#     ScriptType: Customer
#     Load at Startup: [unchecked]
# 2- Include new script in main script
#    Gateway -> Routing Scripts -> [Select main script]
#     2A. Add the "require 'capture_dtmf'" statement at the top of the main script.
#     2B. Add the "include CaptureDTMF" statement in the main routing class.
#     2C. Add the filter "after_filter :method => :capture_dtmf" in the main routing class.
# 3- Upload wav files in gateway
#     Audio Prompts -> Upload audio prompt
#       fen-tone-400hz-300ms.wav
#       dialtone.wav
# 4- Add NAP columns 
#    switch_number  --> Value will match the 2 digit code of the TG (NAP)
#    priority
#    weight
# Then activate the configuration

#
module CaptureDTMF

  #
  # Initialization method
  #
  def init_capture_dtmf( params )

  end

  #
  # Actual filter implementation
  #
  def capture_dtmf( params )

    call   = params[:call]
    bridge = params[:bridge]
    nap_lists = params[:naps]

    # Check if we need to play an announcement to the incoming call
    # Just an example here: Decision is based on calling or called number.
    # A real-life application could load a csv file that contains instrutions
    # on how and when to play announcement prompts
    log_trace 3, "Capture dtmf script"
    log_trace 3, "Called = " + call[:called].inspect
    log_trace 3, "Calling = " + call[:calling].inspect
    log_trace 3, "User Context = " + params[:user_context].inspect
   
   
    #If use_context is nil, this means we have never been through this script
    if params[:user_context].nil?
      
      # Check if called number is this one - start digit collection case
      if call[:called] == "18009994444"
        log_trace 3, "Special called number detected"

        #Delete all routes - we will create a new routing table        
        routes = clone_routes params[:routes]
      
        log_trace 3, "Start detecting DTMF"
     		# Save called number information - This is to keep the called number (it will be erased with the tone detection)
        # Need to do this only the first time (user_context is nil)
		    params[:user_context] = call[ :called ]
        log_trace 3, "Saved Called Number for future use" + params[:user_context].inspect
		
        bridge[ :announcement_tone ]          = "fen-tone-400hz-300ms.wav"
        bridge[ :announcement_tone_answer ]   = "true"
      
        code_detect = { :type => :DTMF, :suffix => "#", :timeout => 5000, :proceed_on_play_done => false, :suffix_removal => true}
        bridge[:announcement_code_detect] = code_detect     
     end  # End find called number      
    
    # We have already saved the called number, now we want to check if the dialed numbers are ok.
    # Check if detected digits (now called number) has a size of 2 --> This is the switch number
    elsif call[:called].length == 2
        switch_number = call[:called]
        log_trace 3, "Switch number is " + switch_number.inspect
        #Save it in user_context
        params[:user_context] = switch_number
       
       # Start a new tone detection
        bridge[ :announcement_tone ]          = "fen-tone-400hz-300ms.wav"
        bridge[ :announcement_tone_answer ]   = "true"
      
        code_detect = { :type => :DTMF, :suffix => "#", :timeout => 5000, :proceed_on_play_done => false, :suffix_removal => true}
        bridge[:announcement_code_detect] = code_detect     
      
    # Check if detected digits (now called number) has a size of 4 or 5 --> This is the Trunk Group number
    elsif (call[:called].length == 4) || (call[:called].length == 5)
        tg_number = call[:called]
        log_trace 3, "Trunk group number is " + tg_number.inspect
        
        #Save it in user_context
        params[:user_context] = params[:user_context] + tg_number
       
       # Start a new tone detection
        bridge[ :announcement_tone ]          = "fen-tone-400hz-300ms.wav"
        bridge[ :announcement_tone_answer ]   = "true"
      
        code_detect = { :type => :DTMF, :suffix => "#", :timeout => 5000, :proceed_on_play_done => false, :suffix_removal => true}
        bridge[:announcement_code_detect] = code_detect     
      
      # Check if detected digits (now called number) has a size of more than 5 --> This is the destination number
    elsif (call[:called].length > 6)
      log_trace 3, "Destination number is " + call[:called].inspect
      
      # Recover switch (2 first digits) and trunk group (the rest) of user_context
      switch_number = params[:user_context][0..1]
      log_trace 3, "Recovered switch number is " + switch_number.inspect
      tg_number  = params[:user_context][2..-1]
      log_trace 3, "Recovered trunk number is " + tg_number.inspect
      
      log_trace 3, "Adding trunk group information in call"
		  call[:request_uri] = call[:request_uri] + ";tgrp=" + tg_number
      call[:request_uri_forward_enabled] = 1
      log_trace 3, "New request uri" + call[:request_uri].inspect
      #Also force the calling number to a new value
      call[:calling] = "4443331234"
      
      newroutes = []

      # Find destination NAP according to tg_number and create new route
      nap_lists.each do |nap_list,nap_info|
        if nap_info[:switch_number] == switch_number
          log_trace 3, "Switch route found!" + nap_info[:name].inspect
          # newroutes << { :nap=>"NAP_SIP_DELL", :route_name => "Test_Route", :name => nap_info[:name], :remapped_nap => nap_info[:name], :priority => nap_info[:priority], :weight => nap_info[:weight], :remapped_calling => call[:calling], :remapped_called => call[:called],:called_noa=>"national_number", :USE_REMAP_CALLING=>"YES", :forward_sip_domain=>"0" }
          newroutes << { :route_name => "Test_Route", :name => nap_info[:name], :remapped_nap => nap_info[:name], :priority => nap_info[:priority], :weight => nap_info[:weight] }
        end #nap_info
      end #nap_lists
      log_trace 3, "New test route:" + newroutes.inspect
      
     # Keep called number
     params[:call] = call 
     params[:routes] = newroutes
      
    else 
      # No need to process: This is a normal call
      log_trace 3, "This is a normal call"
    end
    
    params
  end

end
